# Copyright (C) 2000 Jean-Claude Wippler <jcw@equi4.com>
#
# Updater for the MK-at-end-of-exe datafile (using VFS directory trees)
#
#   This code is used for two purposes:
#       1) to create a new TclKit executable with a datafile at the end
#       2) to update the contents of that datafile with modified files
#
#   When called repeatedly, the datafile may start to accumulate free
#   space.  To get the smallest version, delete the target before use.
#
#   This file is called "setup.tcl", because a freshly compiled TclKit
#   expects to find that in the current dir when it has no datafile yet.
#
#   Output is sent to file "log.txt", because there may not be a stdout.
#
# How to prepare a freshly compiled TclKit(Sh) for use:
#   1)  copy the exe as "kit.exe" into this dir (can be upx-compressed)
#   2)  make sure this script (setup.tcl) and "sync.tcl" are present
#   3)  put all files to be packaged in dirs "kitsh.vfs" and "kit.vfs"
#   4)  puts platform-specific files in dirs "kit[sh]-*.vfs"
#   5)  launch "./kit", the result will be called "tclkit"
# For tclkitsh: follow the same steps, but it'll only use "kitsh.vfs".
#
# 20000303 jcw  Initial version

set compressed 1

set origExe [info nameofexecutable]                 ;# e.g. "C:/.../kit.exe"
set origTail [file tail $origExe]                   ;# e.g. "kit.exe"
set origBase [file rootname $origTail]              ;# e.g. "kit"

set fromDir $origBase.vfs                           ;# e.g. "kit.vfs"
set platDir $origBase-$tcl_platform(platform).vfs   ;# e.g. "kit-unix.vfs"
set destExe "tcl[string tolower $origTail]"         ;# e.g. "tclkit.exe"

set LOG [open log.txt w]
proc tclLog {args} {puts $::LOG [join $args]; flush $::LOG}

# if destination doesn't exist, create a fresh copy

if {![file exists $destExe]} {
    tclLog " creating $destExe from $origExe"
    file copy $origExe $destExe
}

# mount datafile, synchronize directory trees into it, and clean up

catch {load "" pink}
catch {load "" rechan}
catch {load "" vfs}

source kitsh.vfs/lib/vfs/vfs.tcl
source kitsh.vfs/lib/vfs/mk4vfs.tcl

mk4vfs::mount $destExe $destExe

set dirs [list $fromDir $platDir]
if {[string equal -nocase $fromDir kit.vfs]} {
    lappend dirs kitsh.vfs kitsh-$tcl_platform(platform).vfs
}

foreach dir $dirs {
    set argv [list -compress $compressed -auto 0 -noerror 0 $dir $destExe]
    source sync.tcl
}

mk4vfs::umount $destExe

set oSize [file size $origExe]
set nSize [file size $destExe]

tclLog " $origExe: $oSize, $destExe: $nSize"

exit ;# prevent further initialization
